{
 'title': 'Cython Tutorial',
 'paper_abstract': '''
Cython is a programming language based on Python with extra
syntax to provide static type declarations. This takes advantage of the
benefits of Python while allowing one to achieve the speed of C.
In this paper we describe the Cython language and show how it can
be used both to write optimized code and to interface with external
C libraries.
''',
 'authors': [
     {'first_names': 'Stefan',
      'surname': 'Behnel',
      'address': '',
      'country': 'Germany',
      'email_address': 'stefan\_ml@behnel.de',
      'institution': ''},
     {'first_names': 'Robert W.',
      'surname': 'Bradshaw',
      'address': '',
      'country': 'USA',
      'email_address': 'robertwb@math.washington.edu',
      'institution': '''University of Washington\\footnote{
        Department of Mathematics, University of Washington, Seattle, WA, USA
      }'''},
     {'first_names': 'Dag Sverre',
      'surname': 'Seljebotn',
      'address': '',
      'country': 'Norway',
      'email_address': 'dagss@student.matnat.uio.no',
      # I need three institutions w/ full address... leave it
      # all here until we get to editing stage
      'institution': '''University of Oslo\\footnote{Institute of Theoretical Astrophysics,
        University of Oslo, P.O. Box 1029 Blindern, N-0315 Oslo, Norway}\\footnote{Department
        of Mathematics, University of Oslo, P.O. Box 1053 Blindern,
        N-0316 Oslo, Norway}\\footnote{Centre of Mathematics for
        Applications, University of Oslo, P.O. Box 1053 Blindern, N-0316
        Oslo, Norway}'''}
 ],
}
