# mode: compile

cdef struct S:
    char c
    unsigned char uc
    signed char sc
    short s
    unsigned short us
    signed short ss
    int i
    unsigned int ui
    signed int si
    long l
    unsigned long ul
    signed long sl
    long long ll
    unsigned long long ull
    signed long long sll
