# tag: posix
# mode: compile

cimport posix

cimport posix.unistd
from posix cimport unistd
from posix.unistd cimport *

cimport posix.fcntl
from posix cimport fcntl
from posix.fcntl  cimport *

cimport posix.types
from posix cimport types
from posix.types cimport *

cimport posix.signal
from posix cimport signal
from posix.signal cimport *

cimport posix.stat
from posix cimport stat
from posix.stat cimport *

cimport posix.stdio
from posix cimport stdio
from posix.stdio cimport *

cimport posix.stdlib
from posix cimport stdlib
from posix.stdlib cimport *

cimport posix.time
from posix cimport time
from posix.time cimport *

cimport posix.resource
from posix cimport resource
from posix.resource cimport *

cimport posix.wait
from posix cimport wait
from posix.wait cimport *

cimport posix.mman
from posix cimport mman
from posix.mman cimport *

cimport posix.dlfcn
from posix cimport dlfcn
from posix.dlfcn cimport *
